Log("Bubbles Lua script started")

local Bubbles = {}
local randomQuack
local randomLand
local quackTimer
local animationReset
local duckSlide
local duckSurface
local duckDive
local componentSound

function Bubbles.GetTextureList()
	return ""
end

function Bubbles.Init()
	Log("Init")
	OverloadVehicleSounds()
	OverloadPickupSounds()

	quackTimer = 0.0
	animationReset = 0.0
	duckSlide = -1
	duckSurface = -1
	duckDive = -1
	componentSound = 0.0
end

function Bubbles.PickupSequenceUpdate(dt)
	Log("PickupSequenceUpdate")
end

function Bubbles.ComponentComplete()
	Log("Component!")
	componentSound = math.random(3)
	Log("randomQuack = "..componentSound)

	if(componentSound == 1)then
		PlaySound("duck_form_1")
	elseif(componentSound == 2)then
		PlaySound("duck_form_2")
	elseif(componentSound == 3)then
		PlaySound("duck_form_3")	
	end
	componentSound = 0
end

function Bubbles.ConstructionComplete()
	Log("Construction!")
	PlaySound("duck_form_complete")
end

function Bubbles.Ready(skipPickupSeq)
	Log("Ready")
end

function Bubbles.Update(dt)
	if(quackTimer > 0.0)then
		quackTimer = quackTimer - dt
		if(quackTimer < 0.0)then
			quackTimer = 0.0
			BubblesQuack(0)
		end
	end
	
	if(animationReset > 0.0)then
		animationReset = animationReset - dt
		if(animationReset < 0.0)then
			animationReset = 0.0
			PlayAnimation(-1, "idle")
		end
	end
	
end


function Bubbles.Draw()
end

function Bubbles.HitGround()
	if(duckSlide == -1)then
		duckSlide = PlaySound("duck_slide_lp", true)
	end
	Log("HitGround")
end

function Bubbles.LeaveGround()
	StopSound(duckSlide,0.2)
	duckSlide = -1
	Log("LeaveGround")
end

function Bubbles.Dive()
	StopSound(duckSurface,0.2)
	duckSurface = -1
	if(duckDive == -1)then
		duckDive = PlaySound("duck_underwater_lp", true)
	end
	Log("Dive")
end

function Bubbles.Surface()
	StopSound(duckDive,0.2)
	duckDive = -1
	if(duckSurface == -1)then
		duckSurface = PlaySound("duck_surface_lp", true)
	end
	PlaySound("duck_jump")
	Log("Surface")
end

function Bubbles.Airbourne()
	StopSound(duckSurface,0.2)
	duckSurface = -1
	StopSound(duckDive,0.2)
	duckDive = -1
	PlaySound("duck_jump")
	quackTimer = 0.1
	Log("Airbourne")
end

function Bubbles.HitWater()
	if(duckSurface == -1)then
		duckSurface = PlaySound("duck_surface_lp", true)
	end
	Log("duckSurface = "..duckSurface)
	
	randomLand = math.random(4)
	if(randomLand == 1)then
		PlaySound("duck_land")
	elseif(randomLand == 2)then
		PlaySound("duck_land_1")
	elseif(randomLand == 3)then
		PlaySound("duck_land_2")
	elseif(randomLand == 4)then
		PlaySound("duck_land_3")
	end
	
	randomLand = 0
	
	Log("HitWater")
end

function Bubbles.TouchUp()
	inputUp = true
end

function Bubbles.TouchDown()
	inputUp = false
end

function Bubbles.Explode()
	PlaySound("duck_explode")
	PlaySound("duck_impact")
	StopSound(duckDive,0.2)
	duckDive = -1
	StopSound(duckSlide,0.2)
	duckSlide = -1
	StopSound(duckSurface,0.2)
	duckSurface = -1
	Log("Explode")
end

function Bubbles.Destroy(didQuit)

	Log("Destroy")
end

function LogVec(name, vec)
	Log(name..": "..vec[1]..", "..vec[2])
end

function BubblesQuack(a)
	randomQuack = math.random(10)
	Log("randomQuack = "..randomQuack)

	if(a == 0)then
		if(randomQuack == 1)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_1")
			animationReset = 0.5
		elseif(randomQuack == 2)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_2")
			animationReset = 0.5
		elseif(randomQuack == 3)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_3")	
			animationReset = 0.5
		elseif(randomQuack == 4)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_4")
			animationReset = 0.5
		elseif(randomQuack == 5)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_5")	
			animationReset = 0.5
		elseif(randomQuack == 6)then
			PlayAnimation(-1, "quack")
			PlaySound("duck_vox_6")
			animationReset = 0.5
		end
	elseif(a == 1)then
		PlayAnimation(-1, "quack_long")
		PlaySound("duck_form_complete")
		animationReset = 1.0
	end


	randomQuack = 0
end

RegisterScript("Bubbles", Bubbles);