Log("Micro Lua script started")

local Chariot = {}
local headPos
local chariotWing
local animation = ""
local lastAnimationFrame = 0
local startNeigh = false
local playFallingSound = false
local fallingSound
local playGroundSound = false
local groundSound = false
local bodyRotation = 0.0
local neighCounter = 0
local horseOneParticle
local horseTwoParticle
local chariotParticle
local horseOneRainbowParticle
local horseTwoRainbowParticle
local horseOneRainbowTrail
local horseTwoRainbowTrail
local puppyWings
local rainbowSin

function Chariot.GetTextureList()
	return "textures/Characters/Vehicles/Components/Sled/Chariot/componentBackWing.tex|textures/Characters/Vehicles/sled/Chariot/chariotWing.tex|textures/Characters/Vehicles/sled/Chariot/chariotWingBack.tex|textures/Effects/Jetpack/rainbowSparkle.tex|textures/Characters/Vehicles/GravitySuit/MagnetSuit/magnetTransition.tex|textures/Characters/RobotDog/Vehicles/Chariot/sledAntlers.tex"
end

function Chariot.PickupStart()
	OffsetPickupComponents(4,-5)
	OffsetChildPickupComponents(5,0)
	SetFirstComponentToFront(false)
	SetChildComponentToFront(false)
end

function Chariot.ComponentComplete(vehicleType)
	Log("Component!")
	
end

function Chariot.ConstructionComplete(vehicleType)
	if(vehicleType == "Reindeer")then
		if(startNeigh == false)then
			Log("Neigh! "..tostring(vehicleType))
			PlaySound("chariot_form_complete")
			startNeigh = true
			OffsetVehiclePosition(19,-9)
			UpdateVehicle()
			SetBarryOffset(-16,5)
		end
	end
end

function Chariot.Init()
	playGroundSound = false
	startNeigh = false
	rainbowSin = 10
	SetDrawOffset(5,-5)
	SetDeerOffset(2,0)
	SetSleighOffset(-4,-1)
	SetSleighAnimation("easeIn", "")
	SetSleighAnimation("easeOut", "")
	SetSleighAnimation("jump", "jump")
	SetSleighAnimation("land", "land")
	SetDrawBody(true)
	OverloadVehicleSounds()
	OverloadPickupSounds()
	chariotWing = LoadTexture("textures/Characters/Vehicles/Sled/Chariot/ChariotWing.xml", 12)
	chariotWingBack = LoadTexture("textures/Characters/Vehicles/Sled/Chariot/ChariotWingBack.xml", 10)
	puppyWings = LoadTexture("textures/Characters/RobotDog/Vehicles/Chariot/PuppyWings.xml", 14)
	Log("Init Particles!") 
	SetSledEffects(false)
	horseOneRainbowTrail = CreateRainbowTrail(0,0, false, false, rainbowSin)
	horseTwoRainbowTrail = CreateRainbowTrail(0,0, false, false, rainbowSin)
	SetRainbowFadeDist(horseOneRainbowTrail, 60)
	SetRainbowFadeDist(horseTwoRainbowTrail, 60)
	SetRainbowFadeSpeed(horseOneRainbowTrail, 10)
	SetRainbowFadeSpeed(horseTwoRainbowTrail, 10)
	AddPickupComponent("textures/Characters/Vehicles/Components/Sled/Chariot/componentBackWing.tex", 4)
end

function Chariot.PickupSequenceUpdate(dt)
	--Log("PickupSequenceUpdate")
end

function Chariot.Ready(skipPickupSeq)
	PlaySong("Music_Chariot")
	Log("Ready")
	
	if(skipPickupSeq)then
		SetBarryOffset(-16,5)
	end
	
	--SetDeerAnimation("falling",1)
	horseOneParticle = CreateParticle("horse_puff")
	horseTwoParticle = CreateParticle("horse_puff")
	chariotParticle = CreateParticle("horse_puff")
	horseOneRainbowParticle = CreateParticle("horse_rainbow")
	horseTwoRainbowParticle = CreateParticle("horse_rainbow")
	SetParticleEnabled(horseOneParticle, false)
	SetParticleEnabled(horseTwoParticle, false)
	SetParticleEnabled(chariotParticle, false)
	SetParticleEnabled(horseOneRainbowParticle, false)
	SetParticleEnabled(horseTwoRainbowParticle, false)
	
end

function Chariot.Update(dt)
	anim, animationFrame = GetDeerAnimation()
	if(anim == "fly")then

		if(animationFrame == 4 and lastAnimationFrame ~= 4)then
			ChariotFlap()
			SetParticleEnabled(horseOneParticle, true)
			SetParticleEnabled(horseTwoParticle, true)
			SetParticleEnabled(chariotParticle, true)
		end
		if(animationFrame == 6 and lastAnimationFrame ~= 6)then
			SetParticleEnabled(horseOneParticle, false)
			SetParticleEnabled(horseTwoParticle, false)
			SetParticleEnabled(chariotParticle, false)
		end
	elseif(anim == "fall")then
		if(animationFrame == 5 and lastAnimationFrame ~= 5)then	
			ChariotFlap()
		end
	elseif(anim == "jump")then
		if(animationFrame == 1 and lastAnimationFrame ~= 1)then
			ChariotFlap()
			SetParticleEnabled(horseOneParticle, true)
			SetParticleEnabled(horseTwoParticle, true)
			SetParticleEnabled(chariotParticle, true)
		end
		if(animationFrame == 3 and lastAnimationFrame ~= 3)then
			SetParticleEnabled(horseOneParticle, false)
			SetParticleEnabled(horseTwoParticle, false)
			SetParticleEnabled(chariotParticle, false)
		end		
	end
	if(anim ~= "fly" and anim ~= "fall" and anim ~= "jump")then
		SetParticleEnabled(horseOneParticle, false)
		SetParticleEnabled(horseTwoParticle, false)
		SetParticleEnabled(chariotParticle, false)
	end
	anim, lastAnimationFrame = GetDeerAnimation()
	
	chariotAnim, chariotFrame = GetAnimationFrame(-1)
	
	deerX, deerY = GetDeerPosition(0);
	deer2X, deer2Y = GetDeerPosition(1);
	
	deerX = deerX - 5
	deerY = deerY - 5
	deer2X = deer2X - 5
	deer2Y = deer2Y - 5
	
	levelOffsetX, levelOffsetY = GetLevelOffset();
	levelOffset2X, levelOffset2Y = GetLevelOffset();
	
	rot = 0
	if(vehicleRotation ~= nil)then
		rot = math.rad(vehicleRotation)
	end
	
	startOffset = Vector.New(-GetLevelSpeed())
	desiredDir = RotateVector(startOffset, rot)
	finalDir = desiredDir - startOffset
	finalDir[2] = finalDir[2]*0.5
	
	SetRainbowVelocity(horseOneRainbowTrail, desiredDir[1], desiredDir[2])
	SetRainbowVelocity(horseTwoRainbowTrail, desiredDir[1], desiredDir[2])
	
	UpdateRainbowTrail(horseOneRainbowTrail, deerX - levelOffsetX, deerY, true)
	UpdateRainbowTrail(horseTwoRainbowTrail, deer2X - levelOffset2X, deer2Y, true)
	
	neighCounter = neighCounter + dt
	--Log("NeighCounter = "..neighCounter)	
	if(neighCounter > 2)then
		HorseNeigh()
		neighCounter = 0
	end
	
	--Log("Current Chariot Animation = "..tostring(chariotAnim))
	
end

function Chariot.Draw()
	headPos = Vector.New(GetVehiclePosition())
	worldPos = Vector.New(GetLevelOffset())
	
	vehicleRotation = GetVehicleRotation()
	wingRotation = math.rad(vehicleRotation) 
	
	chariotFrontWingPos = Vector.Clone(headPos)
	chariotBackWingPos = Vector.Clone(headPos)
	
	frontWingOffset = Vector.New(-23,2)
	frontWingOffset = RotateVector(frontWingOffset, -wingRotation)
	
	backWingOffset = Vector.New(-20,4)
	backWingOffset = RotateVector(backWingOffset, -wingRotation)
	
	chariotFrontWingPos = chariotFrontWingPos + frontWingOffset
	chariotBackWingPos = chariotBackWingPos + backWingOffset
	
	horseOnePos = Vector.New(0,0)
	horseTwoPos = Vector.New(0,0)
	
	horseOnePos[1], horseOnePos[2] = GetDeerPosition(0)
	horseTwoPos[1], horseTwoPos[2] = GetDeerPosition(1)
	
	chariotParticlePos = headPos + Vector.New(-20,10) - worldPos
	SetParticlePosition(chariotParticle, chariotParticlePos[1], chariotParticlePos[2])
	
	horseOneRainbowPos = horseOnePos + Vector.New(10,10) - worldPos
	horseTwoRainbowPos = horseTwoPos + Vector.New(10,10) - worldPos
	
	horseOnePos = horseOnePos - worldPos
	horseTwoPos = horseTwoPos - worldPos
	
	SetParticlePosition(horseOneParticle, horseOnePos[1], horseOnePos[2])
	SetParticlePosition(horseTwoParticle, horseTwoPos[1], horseTwoPos[2])
	
	SetParticlePosition(horseOneRainbowParticle, horseOneRainbowPos[1], horseOneRainbowPos[2])
	SetParticlePosition(horseTwoRainbowParticle, horseTwoRainbowPos[1], horseTwoRainbowPos[2])
	
	DrawTexture(chariotWing, chariotFrontWingPos[1], chariotFrontWingPos[2], -vehicleRotation)
	DrawTexture(chariotWingBack, chariotBackWingPos[1], chariotBackWingPos[2], -vehicleRotation)
	
	DrawTexture(chariotWingBack, chariotBackWingPos[1], chariotBackWingPos[2], -vehicleRotation)
	
	DrawRainbowTrail(horseOneRainbowTrail)
	DrawRainbowTrail(horseTwoRainbowTrail)
	
	-- PUPPY STUFF!
	if(IsDogActive() == true)then
		dogPosition = Vector.New(GetDogPosition())
		dogRotation = GetDogRotation()
		dogScale = Vector.New(GetDogScale())
	
		if(dogScale[1] < 0)then
			dogScale[1] = -dogScale[1]
		end
		
		if(dogScale[2] < 0)then
			dogRotation = dogRotation + 180
			dogScale[1] = -dogScale[1]
			dogScale[2] = -dogScale[2]
		end
		
		dogOffset = Vector.New(0.0,0.0)
		dogOffset = RotateVector(dogOffset, math.rad(dogRotation))
		dogPosition = dogPosition + dogOffset
		DrawTexture(puppyWings, dogPosition[1], dogPosition[2], dogRotation, dogScale[1], dogScale[2])
	end
end

function Chariot.HitRoof()
	Log("HitRoof")
end

function Chariot.HitGround()
	Log("HitGround")
end

function Chariot.TouchUp()
	inputUp = true

end

function Chariot.TouchDown()
	if(playGroundSound == true)then
		Log("Stop Ground TouchDown")
		StopSound(groundSound)
		playGroundSound = false
	end
	if(playFallingSound == true)then
		StopSound(fallingSound,0.2)
		playFallingSound = false
	end

end

function Chariot.Diving()
	wingAnim, wingFrame = GetAnimationFrame(chariotWing)

	PlayAnimation(chariotWing,"flap_slow",wingFrame)
	PlayAnimation(chariotWingBack,"flap_slow",wingFrame)
	if(playGroundSound == true)then
		Log("Stop Ground Sound Diving")
		StopSound(groundSound)
		playGroundSound = false
	end
	if(playFallingSound == false)then
		fallingSound = PlaySound("whale_flyLP",true)
		playFallingSound = true
	end
	--Log("Diving, animationFrame = "..animationFrame)
end

function Chariot.Flying()
	wingAnim, wingFrame = GetAnimationFrame(chariotWing)
	anim, animFrame = GetAnimationFrame(-1)
	if(anim == "chariot_spin")then
		PlayAnimation(-1,"chariot_slow")
	end
	PlayAnimation(chariotWing,"flap",wingFrame)
	PlayAnimation(chariotWingBack,"flap",wingFrame)
	if(playGroundSound == true)then
		Log("Stop Ground Sound Flying")
		StopSound(groundSound)
		playGroundSound = false
	end
	if(playFallingSound == true)then
		StopSound(fallingSound,0.2)
		playFallingSound = false
	end
	
	--Log("Flying animationFrame = "..animationFrame)	
end

function Chariot.Walking()
	wingAnim, wingFrame = GetAnimationFrame(chariotWing)
	PlaySound("chariot_land")
	PlayAnimation(chariotWing,"idle")
	PlayAnimation(chariotWingBack,"idle")
	if(playGroundSound == false)then
		groundSound = PlaySound("chariot_lp", true)
		playGroundSound = true
	end		
	if(playFallingSound == true)then
		StopSound(fallingSound,0.2)
		playFallingSound = false
	end
	PlayAnimation(-1,"chariot_spin")

end

function Chariot.Explode()
	Log("Explode")
end

function Chariot.Destroy(didQuit)
	if(didQuit == false)then
		StopSong()
		PlaySound("chariot_death")
		PlaySong("default", 2.0)

	else
		PlaySong("default")
	end

	StopSound(fallingSound,0.2)
	playFallingSound = false
	Log("Stop Ground Sound Quit")
	StopSound(groundSound)
	playGroundSound = false
	SetParticleEnabled(horseOneRainbowParticle, false)
	SetParticleEnabled(horseTwoRainbowParticle, false)
	Log("Destroy")
end

function LogVec(name, vec)
	Log(name..": "..vec[1]..", "..vec[2])
end

function ChariotFlap()
	chariotFlap = math.random(3)

	if(chariotFlap == 1)then
		PlaySound("chariot_flap_01")
	elseif(chariotFlap == 2)then
		PlaySound("chariot_flap_02")
	elseif(chariotFlap == 3)then
		PlaySound("chariot_flap_03")	
	end

	chariotFlap = 0
end

function HorseNeigh() 
	neigh = math.random(10)

	if(neigh == 1)then
		PlaySound("chariot_vox_01")
	elseif(neigh == 2)then
		PlaySound("chariot_vox_02")
	elseif(neigh == 3)then
		PlaySound("chariot_vox_03")	
	elseif(neigh == 4)then
		PlaySound("chariot_vox_04")	
	end

	neigh = 0
end

function RotateVector(vec, rot)
	x = math.cos(rot) * vec[1] - math.sin(rot) * vec[2];
	y = math.sin(rot) * vec[1] + math.cos(rot) * vec[2];
	retVec = Vector.New(x, y)
	return retVec;
end

function RotateOffset(orig, pos, rot)
	offset = pos - orig
	offset = RotateVector(offset, rot)
	pos = orig + offset
	return pos
end


function Chariot.FlashToAnim_jumpLaunch(startFrame)
	Log("Flash JumpLaunch! Frame: "..startFrame)	
	PlayAnimation(puppyWings, "jump", startFrame)

end

function Chariot.FlashToAnim_land(startFrame)
	Log("Flash Land! Frame: "..startFrame)
	PlayAnimation(puppyWings, "land", startFrame)

end

function Chariot.FlashToAnim_run(startFrame)
	Log("Flash Run! Frame: "..startFrame)
	PlayAnimation(puppyWings, "run", startFrame)

end

RegisterScript("Chariot", Chariot);