Log("Washer Lua script started")

local Washer = {}
local animationReset
local idleSound
local teleporterTimer
local washerBack
local headPos
local componentSound
local backRotation = 0
local playerRotation = 0
local washerBubbles
local washerSocks

function Washer.GetTextureList()
	return "textures/Characters/Vehicles/Components/Teleporter/Washer/ComponentBack.tex"
end

function Washer.ComponentComplete()
	Log("Component!")
	componentSound = math.random(3)

	if(componentSound == 1)then
		PlaySound("vehicle_form_1")
	elseif(componentSound == 2)then
		PlaySound("vehicle_form_2")
	elseif(componentSound == 3)then
		PlaySound("vehicle_form_3")	
	end
	componentSound = 0
end

function Washer.ConstructionComplete()
	Log("Construction!")
	PlaySound("box_form_complete")
end


function Washer.Init()
	Log("Init")
	SetDrawBody(false)
	OverloadVehicleSounds()
	OverloadPickupSounds()
	animationReset = 0.0
	idleSound = -1
	teleporterTimer = 0.0
	washerBack = LoadTexture("textures/Characters/Vehicles/Teleporter/Washer/washerBacking.xml", 9)
	SetLighteningOffset(15, 20)
	AddPickupComponent("textures/Characters/Vehicles/Components/Teleporter/Washer/ComponentBack.tex", 9, -1)
	
	washerBubbles = CreateParticle("Washer_Bubbles")
	SetParticleEnabled(washerBubbles, false)
	washerSocks = CreateParticle("Washer_Socks")
	SetParticleEnabled(washerSocks, false)
	
end

function Washer.PickupSequenceUpdate(dt)
	Log("PickupSequenceUpdate")
end

function Washer.Ready(skipPickupSeq)
	Log("Ready")
	idleSound = PlaySound("box_lp",true)
	PlayAnimation(washerBack, "idle")
end

function Washer.Update(dt)

	backRotation = backRotation + 0.1
	playerRotation = playerRotation + 0.2
	if(backRotation > 360)then
		backRotation = 0
	end

	if(playerRotation > 360)then
		playerRotation = 0
	end

end  

function Washer.Draw()
	headPos = Vector.New(GetPlayerPosition())
	worldPos = Vector.New(GetLevelOffset())
	
	washerSocksPos = headPos - worldPos
	
	SetParticlePosition(washerSocks, washerSocksPos[1], washerSocksPos[2] + 10)
	SetParticlePosition(washerBubbles, headPos[1], headPos[2] + 10)
	
	anim, animationFrame = GetAnimationFrame(-1)
	
	if(anim == "idle")then
		DrawTexture(washerBack, headPos[1], headPos[2] + 10, math.deg(-backRotation))
		--DrawPlayer(headPos[1], headPos[2] + 10, math.deg(playerRotation),math.deg(playerRotation))
	elseif(anim == "teleport_out" and animationFrame == 2)then
		SetParticleEnabled(washerBubbles, true)
		sockGen = math.random(4)
		Log("Bubbles!")
		if(sockGen == 1)then
			Log("Socks!")
			SetParticleEnabled(washerSocks, true)
		end
	elseif(anim == "teleport_out" and animationFrame == 4)then
		SetParticleEnabled(washerBubbles, false)
		SetParticleEnabled(washerSocks, false)
	end
	

end

function Washer.HitRoof()
	Log("HitRoof")
end

function Washer.HitGround()
	Log("HitGround")
end

function Washer.TouchUp()

	Log("TouchUp")
	inputUp = true
end

function Washer.TouchDown()
	Log("TouchDown")

	inputUp = false
end

function Washer.TeleportOut()
	--if(teleporterTimer == 0)then
		PlaySound("box_activate")
		--teleporterTimer = 0.4
		--PlayAnimation(washerBack, "teleport_out")
		--Log("washerBack TouchDown ="..washerBack)
	--end
	Log("Teleport Out")
end

function Washer.TeleportIn()
	--PlayAnimation(washerBack, "teleport_in")
	teleporterTimer = 0.2
	Log("Teleport In")
end

function Washer.Explode()
	StopSound(idleSound,0.2)
	idleSound = -1
	PlaySound("box_explode")
	Log("Explode")
end

function Washer.Destroy(didQuit)
	Log("Destroy")
	if(idleSound >= 0)then
		StopSound(idleSound,0.2)
		idleSound = -1
	end
end

function LogVec(name, vec)
	Log(name..": "..vec[1]..", "..vec[2])
end

RegisterScript("Washer", Washer);