Log("ZombieTrex Lua script started")


local ZombieTrex = {}
local rocketFlame
local headPos = 0
rocketTimer = 0.0
local running = false
local lastAnimationFrame = 0
local runDustLeft
local runDustRight
local jumpDust
local landDust
local leftCounter = 0.0
local leftMove = 0.0
local rightMove = 0.0
local rightCounter = 0.0
local moveJump = 0.0
local moveLand = 0.0
local jumpCounter = 0.0
local landCounter = 0.0
local flutterSound = -1
local playFlutter = -1

function ZombieTrex.GetTextureList()
	return "textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentTail.tex|textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentBelt.tex|textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentBody.tex|textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentHead.tex|textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentLeg.tex|textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentLegFront.tex|textures/Characters/Vehicles/Mech/ZombieTrex/vehicleMechFlame.tga"
end

function ZombieTrex.ComponentComplete()

	componentSound = math.random(3)

	if(componentSound == 1)then
		PlaySound("trex_form_1")
	elseif(componentSound == 2)then
		PlaySound("trex_form_2")
	elseif(componentSound == 3)then
		PlaySound("trex_form_3")
	end
	componentSound = 0
end

function ZombieTrex.ConstructionComplete()
	Log("Construction!")
	PlaySound("trex_form_complete")
end


function ZombieTrex.Init()
	playFlutter = -1
	OverloadVehicleSounds()
	OverloadPickupSounds()
	SetDrawBody(true)
	SetMechEffects(false)
	OverloadAllowHalo(true)
	runDustLeft = LoadTexture("textures/Effects/runDust.xml", 9)
	runDustRight = LoadTexture("textures/Effects/runDust.xml", 9)
	jumpDust = LoadTexture("textures/Effects/jumpDust.xml", 9)
	landDust = LoadTexture("textures/Effects/landDust.xml", 9)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentBody.tex", 11, 1)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentLeg.tex", 9, 1)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentLegFront.tex", 11, 1)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentTail.tex", 11, 1)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentHead.tex", 9, 1)
	AddPickupComponent("textures/Characters/Vehicles/Components/Mech/ZombieTrex/ComponentBelt.tex", 11, 1)
	
	--set a huge piece number so his body doesn't disappear
	SetConstructionBodyDisappearPieceNumber(99);
end

function ZombieTrex.PickupSequenceUpdate(dt)
	Log("PickupSequenceUpdate")
end

function ZombieTrex.Ready(skipPickupSeq)
	Log("Ready")
end

function ZombieTrex.Update(dt)
	if(running == true)then
		anim, animationFrame = GetAnimationFrame(-1)
		if(animationFrame == 2 and lastAnimationFrame ~= 2)then
			PlaySound("trex_step_1")
			PlayAnimation(runDustLeft, "runDino")
			ScreenShake(0.2,0.2)
			leftCounter = 0.4
			leftMove = 0.0
		elseif(animationFrame == 6 and lastAnimationFrame ~= 6)then
			PlaySound("trex_step_2")
			ScreenShake(0.2,0.2)
			TrexRoar()
			PlayAnimation(runDustRight, "runDino")
			rightCounter = 0.4
			rightMove = 0.0
		end
	end
	anim, lastAnimationFrame = GetAnimationFrame(-1)
	
	if(leftCounter > 0.0)then
		leftCounter = leftCounter - dt
		leftMove = leftMove + 2
		--Log("LeftMove "..leftMove)
	end	
	if(leftCounter < 0.0)then
		leftCounter = 0.0
	end
	if(rightCounter > 0.0)then
		rightCounter = rightCounter - dt
		rightMove = rightMove + 2
		--Log("RightMove "..rightMove)
	end	
	if(rightCounter < 0.0)then
		rightCounter = 0.0
	end

	if(jumpCounter > 0.0)then
		jumpCounter = jumpCounter - dt
		moveJump = moveJump + 2
	end	
	if(jumpCounter < 0.0)then
		jumpCounter = 0.0
		moveJump = 0.0
	end
	
	if(landCounter > 0.0)then
		landCounter = landCounter - dt
		moveLand = moveLand + 2
	end	
	if(landCounter < 0.0)then
		landCounter = 0.0
		moveLand = 0.0
	end
	
end

function ZombieTrex.Draw()
	headPos = Vector.New(GetPlayerPosition())
	anim, runDustLeftFrame = GetAnimationFrame(runDustLeft)
	anim, runDustRightFrame = GetAnimationFrame(runDustRight)
	anim, jumpDustFrame = GetAnimationFrame(jumpDust)
	anim, landDustFrame = GetAnimationFrame(landDust)
	
	
	
	if(runDustLeftFrame < 11)then
		DrawTexture(runDustLeft, (headPos[1] + 10 - leftMove), -110)
	end
	if(runDustRightFrame < 11)then
		DrawTexture(runDustRight, (headPos[1] - rightMove), -115)
	end
	if(jumpDustFrame < 5)then
		DrawTexture(jumpDust, (headPos[1] - moveJump), -70)
	end
	if(landDustFrame < 3)then
		DrawTexture(landDust, (headPos[1] - moveLand), -135)
	end
	
end


function ZombieTrex.HitGround()
	Log("HitGround")
	PlaySound("trex_land")
	PlayAnimation(landDust, "land")
	ScreenShake(0.2,5)
	landCounter = 0.4
	if(playFlutter == 1)then
		StopSound(flutterSound)
		Log("Stop Sound!!")
		playFlutter = -1
	end
	running = true
end

function ZombieTrex.OnJump()
	running = false
	PlaySound("trex_jump")
	PlayAnimation(jumpDust, "jump")
	jumpCounter = 0.4
	Log("Jump!")
end

function ZombieTrex.TouchUp()
	inputUp = true
end

function ZombieTrex.TouchDown()

		inputUp = false
end

function ZombieTrex.StartBoost()
	TrexRoar()
	if(playFlutter == -1)then
		Log("Play Sound!!")
		flutterSound = PlaySound("trex_flutter_lp",true)
		playFlutter = 1
	end
	Log("StartBoost")
	Log("StartBoost playFlutter= "..playFlutter)
end

function ZombieTrex.StopBoost()
	Log("StopBoostPre playFlutter= "..playFlutter)
	if(playFlutter == 1)then
		StopSound(flutterSound)
		Log("Stop Sound!!")
		playFlutter = -1
	end
	Log("StopBoostPost playFlutter= "..playFlutter)
	Log("StopBoost")
end

function ZombieTrex.FlameOn()
	Log("FlameOn")
end

function ZombieTrex.FlameOff()
	Log("FlameOff")
end

function ZombieTrex.Explode()
	Log("Explode")
	if(playFlutter == 1)then
		Log("Stop Sound!!")
		StopSound(flutterSound,0.1)
		playFlutter = -1
	end
end

function ZombieTrex.Destroy()
	Log("Destroy")
	PlaySound("trex_death")
end

function LogVec(name, vec)
	Log(name..": "..vec[1]..", "..vec[2])
end

function TrexRoar()
	randomTrex = math.random(8)
	Log("randomTrex = "..randomTrex)

	if(randomTrex == 1)then
		PlaySound("trex_roar_1")
	elseif(randomTrex == 2)then
		PlaySound("trex_roar_2")
	elseif(randomTrex == 3)then
		PlaySound("trex_roar_3")	
	end

	randomTrex = 0
end

function ZombieTrex.FlashToAnim_land()

end

function ZombieTrex.FlashToAnim_run()

end

function ZombieTrex.FlashToAnim_jumpLaunch()

end
RegisterScript("ZombieTrex", ZombieTrex);